/**
 * App Component
 */
import React from 'react';
import M from 'immutable';
import {connect} from 'react-redux';
// import FormBuilder from './FormBuilder';
import FormGenerator from './FormGenerator';
import schema from './schemaNested';
import { Button, Icon } from 'antd';

import brace from 'brace';
import AceEditor from 'react-ace';

import 'brace/mode/json';
import 'brace/theme/github';

const App = React.createClass({
    getInitialState() {
        return {
            schema: schema,
            schemaString: JSON.stringify(schema, null, 4),
            parseError: null
        }
    },
    render() {
        const schema = this.state.schema;
        const schemaString = this.state.schemaString;

        const initialValue = {
            eat: true,
            sex: 'male',
            country: 'China'
        }

        let $form = null;

        if (this.state.parseError) {
            $form = <p>{this.state.parseError.toString()}</p>
        } else {
            $form = <FormGenerator 
                    schema={schema} 
                    initialValue={initialValue}
                    onSubmit={this.onSubmit}/>
        }

        return (
            <div className="app-container">
                <div className="app-navigator">
                    <h1>表单生成器</h1>
                </div>
                <div className="form-generator-wraper">
                    <div className="left">
                        <Button 
                            type="primary" 
                            shape="circle"
                            onClick={this.onUpdateSchema}>
                            <Icon type="reload" />
                        </Button>
                        <AceEditor
                            mode="json"
                            theme="github"
                            width="100%"
                            height="calc(100% - 20px)"
                            showGutter={false}
                            value={schemaString}
                            onChange={this.onChangeSchema}
                            name="UNIQUE_ID_OF_DIV"
                            editorProps={{$blockScrolling: true}}
                          />
                    </div>
                    <div className="right">
                        {$form}
                    </div>
                </div>
            </div>
        )
    },
    onUpdateSchema() {
        try {
            const schemaString = this.state.schemaString;
            const schema = JSON.parse(schemaString);
            this.setState({
                schema,
                parseError: null
            });
        } catch(e) {
            this.setState({
                parseError: e
            })
            throw new Error(e);
        }
    },
    onChangeSchema(value) {
        const schemaString = value;
        this.setState({
            schemaString
        })
    },
    onSubmit(values) {
        console.log(values);
    }
});

/**
 * transform store data to App props
 * @param  {[type]} state [description]
 * @return {[type]}       [description]
 */
const mapStateToProps = (state) => {
	return {
		name: state.get('name'),
		formItems: state.get('formItems')
	}
}

/**
 * add dispatch to 
 * @param  {[type]} dispatch [description]
 * @return {[type]}          [description]
 */
const mapDispatchToProps = (dispatch) => {
	return {
		dispatch: (type, payload) => {
			dispatch({
                type,
                payload
            });
		}
	}
}

/**
 * create redux container
 * @type {[type]}
 */
const AppContainer = connect(mapStateToProps, mapDispatchToProps)(App);

export default AppContainer;